import enum

import numpy as np
import pyqtgraph as pg
from pyqtgraph.Qt import QtCore, QtGui

from .functions import floodfill, getPath


class RectROI(pg.ROI):
    def __init__(self, size, parent=None):
        super().__init__(pos=[0, 0], size=size, parent=parent)
        self.handleSize = 15
        self.addScaleHandle([1, 1], [0, 0])
        self.addRotateHandle([0, 0], [0.5, 0.5])


class PolygonROI(pg.PolyLineROI):
    def __init__(self, positions, closed=False, pos=None, **kwargs):
        if pos is None:
            pos = [0, 0]
        self.closed = closed
        self.segments = []
        pg.ROI.__init__(self, pos, size=[1, 1], **kwargs)
        self.handleSize = 10

        self.setPoints(positions)

    def getArrayRegion(self, arr, img=None, axes=(0, 1),
                       returnMappedCoords=False, **kwds):
        """
        Fix a bug in pg.PolygonROI
        https://github.com/pyqtgraph/pyqtgraph/pull/618
        """
        if returnMappedCoords:
            sliced, mappedCoords = pg.ROI.getArrayRegion(self, arr, img, axes,
                                                         returnMappedCoords,
                                                         fromBoundingRect=True,
                                                         **kwds)
        else:
            sliced = pg.ROI.getArrayRegion(self, arr, img, axes,
                                           returnMappedCoords,
                                           fromBoundingRect=True, **kwds)

        mask = self.renderShapeMask(sliced.shape[axes[0]],
                                    sliced.shape[axes[1]])
        if img.axisOrder != 'col-major':
            mask = mask.T

        # reshape mask to ensure it is applied to the correct data axes
        shape = [1] * arr.ndim
        shape[axes[0]] = sliced.shape[axes[0]]
        shape[axes[1]] = sliced.shape[axes[1]]
        mask = mask.reshape(shape)

        if returnMappedCoords:
            return sliced * mask, mappedCoords * mask
        else:
            return sliced * mask


class EllipseROI(pg.EllipseROI):
    def __init__(self, pos, size, **kwargs):
        pg.ROI.__init__(self, pos, size, **kwargs)
        self.handleSize = 10
        self.addRotateHandle([1.0, 0.5], [0.5, 0.5])
        self.addScaleHandle([0.5*2.**-0.5 + 0.5, 0.5*2.**-0.5 + 0.5],
                            [0.5, 0.5])

    def getArrayRegion(self, arr, img=None, axes=(0, 1),
                       returnMappedCoords=False, **kwds):
        """
        Fix a bug in pg.EllipseROI
        https://github.com/pyqtgraph/pyqtgraph/pull/618
        """
        if returnMappedCoords:
            arr, mappedCoords = pg.ROI.getArrayRegion(self, arr, img, axes,
                                                      returnMappedCoords,
                                                      **kwds)
        else:
            arr = pg.ROI.getArrayRegion(self, arr, img, axes,
                                        returnMappedCoords, **kwds)
        if arr is None or arr.shape[axes[0]] == 0 or arr.shape[axes[1]] == 0:
            if returnMappedCoords:
                return arr, mappedCoords
            else:
                return arr
        w = arr.shape[axes[0]]
        h = arr.shape[axes[1]]

        # generate an ellipsoidal mask
        mask = np.fromfunction(lambda x, y: (((x+0.5)/(w/2.)-1)**2 +
                                             ((y+0.5)/(h/2.)-1)**2)**0.5 < 1,
                               (w, h))

        # reshape to match array axes
        if axes[0] > axes[1]:
            mask = mask.T
        shape = [(n if i in axes else 1) for i, n in enumerate(arr.shape)]
        mask = mask.reshape(shape)
        if returnMappedCoords:
            return arr * mask, mappedCoords * mask
        else:
            return arr * mask


class SemiAutomaticROI(pg.ROI):
    """
    Use floodfill algorithm to make the countour around a bright object.

    The whole ROI can only be moved by selecting the starting point of the
    floodfill algorithm. This starting point is a rectangular handle as defined
    in the *__init__* method.
    """
    def __init__(self, pos=(0, 0), threshold=0.9, parent=None):
        super().__init__(pos=pos, parent=parent, movable=False)
        self.handleSize = 10
        self.addHandle({'name': 'startNode', 'type': 't', 'pos': pos})
        self.threshold = threshold

    def setThreshold(self, threshold):
        self.threshold = threshold
        # Emit change in ROI and force a re-paint
        self.sigRegionChanged.emit(self)
        self.update()

    def getArrayRegion(self, arr, img, axes=(0, 1), returnMappedCoords=False):
        xStart = int(self.state['pos'][0])
        yStart = int(self.state['pos'][1])
        if (xStart < 0 or xStart > img.image.shape[0] or
                yStart < 0 or yStart > img.image.shape[1]):
            mask = np.zeros((img.image.shape[0], img.image.shape[1]),
                            dtype=bool)
        else:
            startVal = img.image[xStart, yStart]
            mask = floodfill(img.image, self.state['pos'],
                             lambda val: val >= startVal * self.threshold)
        if returnMappedCoords:
            mappedCoords = set()
            for i, line in enumerate(mask):
                for j, _ in enumerate(line):
                    mappedCoords.add((i, j))
            self.mask = mask
            return arr * mask, mappedCoords
        else:
            return arr * mask

    def boundingRect(self):
        """
        Return the bounding rectangle of the ROI. The bounding rectangle of the
        ROI is set as the full image (the biggest size the ROI can be) to work
        around what seems to be PyQt painting bug.
        It seems that objects painted outside the bounding rectangle are not
        correctly deleted during a re-paint.

        The bounding rectangle is returned via the mask of the ROI, which is
        only computed after *getArrayRegion* with *returnMappedCoords=True* in
        run.
        """
        if not hasattr(self, 'mask'):
            return QtCore.QRectF()
        else:
            rect = QtCore.QRectF(0, 0, self.mask.shape[0], self.mask.shape[1])
            rect.translate(-self.state['pos'][0], -self.state['pos'][1])
            return rect

    def paint(self, p, opt, widget):
        """
        Paint the path of the ROI using *self.mask*. *self.mask* is only
        computed after *getArrayRegion* with *returnMappedCoords=True* in run.
        """
        if not hasattr(self, 'mask'):
            return
        p.translate(-self.state['pos'][0], -self.state['pos'][1])
        p.setRenderHint(QtGui.QPainter.Antialiasing)
        p.setPen(self.currentPen)

        p.drawPath(getPath(self.mask))


class ROIType(enum.Enum):
    ELLIPSE = EllipseROI
    POLYGON = PolygonROI
    RECTANGLE = RectROI
    SEMIAUTOMATIC = SemiAutomaticROI


class MultiROI(QtGui.QGraphicsObject):
    """
    Container for several type of ROIs.
    This class in itself is only defining the interface between the currently
    selected ROI type and the outside world.

    A few abstract methods from QGraphicsObject must be overridden
    (*boundingRect* and *paint*) for integration with Qt.
    """
    sigRegionChangeFinished = QtCore.Signal(object)
    sigRegionChangeStarted = QtCore.Signal(object)
    sigRegionChanged = QtCore.Signal(object)

    def __init__(self):
        super().__init__()
        self.currentRoi = None
        self.cachedRois = {}
        self.setROIType(ROIType.RECTANGLE)

        # Wrap methods from *currentRoi*
        for fnName in ['setZValue', 'zValue', 'scene', 'setPen',
                       'getArrayRegion']:
            def _create_method(name):
                def method(*args, **kwargs):
                    fn = getattr(self.currentRoi, name)
                    return fn(*args, **kwargs)
                method.__name__ = name
                return method
            setattr(self, fnName, _create_method(fnName))
        del _create_method

    def boundingRect(self):
        return QtCore.QRectF()

    def paint(self, *args):
        pass

    def setROIType(self, roiType):
        self._disconnectAll()

        if roiType in self.cachedRois:
            self.currentRoi = self.cachedRois[roiType]
        else:
            if roiType == ROIType.ELLIPSE:
                self.currentRoi = EllipseROI([10, 10], [10, 10], parent=self)
            elif roiType == ROIType.POLYGON:
                self.currentRoi = PolygonROI([[0, 0], [5, 0],
                                              [6, 6], [0, 5]],
                                             closed=True, parent=self)
            elif roiType == ROIType.RECTANGLE:
                self.currentRoi = RectROI(10, parent=self)
            elif roiType == ROIType.SEMIAUTOMATIC:
                self.currentRoi = SemiAutomaticROI(parent=self)
            else:
                raise NotImplemented('Unhandled ROIType: %s' % repr(roiType))
            self.cachedRois[roiType] = self.currentRoi

        # Hide all ROIs expect the selected type
        for cachedRoiType, cachedRoi in self.cachedRois.items():
            if cachedRoiType == roiType:
                cachedRoi.show()
            else:
                cachedRoi.hide()

        self._connectAll()
        self.sigRegionChanged.emit(self)

    def _disconnectAll(self):
        if self.currentRoi is not None:
            self.currentRoi.sigRegionChanged.disconnect(self.roiChangedEvent)
            self.currentRoi.sigRegionChangeStarted.disconnect(
                self.roiChangeStartedEvent)
            self.currentRoi.sigRegionChangeFinished.disconnect(
                self.roiChangeFinishedEvent)

    def _connectAll(self):
        self.currentRoi.sigRegionChanged.connect(self.roiChangedEvent)
        self.currentRoi.sigRegionChangeStarted.connect(
            self.roiChangeStartedEvent)
        self.currentRoi.sigRegionChangeFinished.connect(
            self.roiChangeFinishedEvent)

    def setSliderValue(self, value):
        if isinstance(self.currentRoi, ROIType.SEMIAUTOMATIC.value):
            self.currentRoi.setThreshold(value)

    def roiChangedEvent(self):
        self.sigRegionChanged.emit(self)

    def roiChangeStartedEvent(self):
        self.sigRegionChangeStarted.emit(self)

    def roiChangeFinishedEvent(self):
        self.sigRegionChangeFinished.emit(self)



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'aflak/FitsHeaderForm.ui'
#
# Created by: PyQt5 UI code generator 5.9.2
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_Form(object):
    def setupUi(self, Form):
        Form.setObjectName("Form")
        Form.resize(400, 300)
        self.gridLayout = QtWidgets.QGridLayout(Form)
        self.gridLayout.setObjectName("gridLayout")
        self.plainTextEdit = QtWidgets.QPlainTextEdit(Form)
        font = QtGui.QFont()
        font.setFamily("Monospace")
        self.plainTextEdit.setFont(font)
        self.plainTextEdit.setTextInteractionFlags(QtCore.Qt.TextSelectableByKeyboard|QtCore.Qt.TextSelectableByMouse)
        self.plainTextEdit.setObjectName("plainTextEdit")
        self.gridLayout.addWidget(self.plainTextEdit, 0, 0, 1, 1)

        self.retranslateUi(Form)
        QtCore.QMetaObject.connectSlotsByName(Form)

    def retranslateUi(self, Form):
        _translate = QtCore.QCoreApplication.translate
        Form.setWindowTitle(_translate("Form", "Form"))




# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'aflak/AboutDialog.ui'
#
# Created by: PyQt5 UI code generator 5.9.2
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_AboutDialog(object):
    def setupUi(self, AboutDialog):
        AboutDialog.setObjectName("AboutDialog")
        AboutDialog.resize(759, 480)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(AboutDialog.sizePolicy().hasHeightForWidth())
        AboutDialog.setSizePolicy(sizePolicy)
        AboutDialog.setLayoutDirection(QtCore.Qt.LeftToRight)
        self.gridLayout = QtWidgets.QGridLayout(AboutDialog)
        self.gridLayout.setObjectName("gridLayout")
        self.label_2 = QtWidgets.QLabel(AboutDialog)
        font = QtGui.QFont()
        font.setPointSize(8)
        self.label_2.setFont(font)
        self.label_2.setObjectName("label_2")
        self.gridLayout.addWidget(self.label_2, 3, 0, 1, 1)
        self.aflakName = QtWidgets.QLabel(AboutDialog)
        font = QtGui.QFont()
        font.setPointSize(18)
        font.setBold(True)
        font.setWeight(75)
        self.aflakName.setFont(font)
        self.aflakName.setAlignment(QtCore.Qt.AlignCenter)
        self.aflakName.setWordWrap(True)
        self.aflakName.setObjectName("aflakName")
        self.gridLayout.addWidget(self.aflakName, 1, 0, 1, 3)
        self.label = QtWidgets.QLabel(AboutDialog)
        font = QtGui.QFont()
        font.setPointSize(8)
        self.label.setFont(font)
        self.label.setObjectName("label")
        self.gridLayout.addWidget(self.label, 3, 1, 1, 1)
        self.aflakAcronym = QtWidgets.QLabel(AboutDialog)
        font = QtGui.QFont()
        font.setPointSize(20)
        font.setBold(True)
        font.setWeight(75)
        self.aflakAcronym.setFont(font)
        self.aflakAcronym.setAlignment(QtCore.Qt.AlignCenter)
        self.aflakAcronym.setObjectName("aflakAcronym")
        self.gridLayout.addWidget(self.aflakAcronym, 0, 0, 1, 3)
        self.closeButton = QtWidgets.QPushButton(AboutDialog)
        self.closeButton.setMaximumSize(QtCore.QSize(100, 16777215))
        self.closeButton.setLayoutDirection(QtCore.Qt.LeftToRight)
        self.closeButton.setObjectName("closeButton")
        self.gridLayout.addWidget(self.closeButton, 4, 0, 1, 1)
        self.versionLabel = QtWidgets.QLabel(AboutDialog)
        font = QtGui.QFont()
        font.setPointSize(16)
        self.versionLabel.setFont(font)
        self.versionLabel.setObjectName("versionLabel")
        self.gridLayout.addWidget(self.versionLabel, 2, 1, 1, 2)
        self.versionFixed = QtWidgets.QLabel(AboutDialog)
        font = QtGui.QFont()
        font.setPointSize(16)
        self.versionFixed.setFont(font)
        self.versionFixed.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.versionFixed.setObjectName("versionFixed")
        self.gridLayout.addWidget(self.versionFixed, 2, 0, 1, 1)

        self.retranslateUi(AboutDialog)
        QtCore.QMetaObject.connectSlotsByName(AboutDialog)

    def retranslateUi(self, AboutDialog):
        _translate = QtCore.QCoreApplication.translate
        AboutDialog.setWindowTitle(_translate("AboutDialog", "About aflak"))
        self.label_2.setText(_translate("AboutDialog", "Developed by "))
        self.aflakName.setText(_translate("AboutDialog", "Advanced Framework for Learning Astrophysical Knowledge"))
        self.label.setText(_translate("AboutDialog", "Malik Olivier Boussejra <malik@boussejra.com>"))
        self.aflakAcronym.setText(_translate("AboutDialog", "aflak"))
        self.closeButton.setText(_translate("AboutDialog", "Close"))
        self.versionLabel.setText(_translate("AboutDialog", "X.X.X"))
        self.versionFixed.setText(_translate("AboutDialog", "Version "))




# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'aflak/FitsHeaderWindow.ui'
#
# Created by: PyQt5 UI code generator 5.9.2
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_FitsHeaderWindow(object):
    def setupUi(self, FitsHeaderWindow):
        FitsHeaderWindow.setObjectName("FitsHeaderWindow")
        FitsHeaderWindow.resize(1350, 900)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(FitsHeaderWindow.sizePolicy().hasHeightForWidth())
        FitsHeaderWindow.setSizePolicy(sizePolicy)
        FitsHeaderWindow.setLayoutDirection(QtCore.Qt.LeftToRight)
        self.gridLayout = QtWidgets.QGridLayout(FitsHeaderWindow)
        self.gridLayout.setObjectName("gridLayout")
        self.label = QtWidgets.QLabel(FitsHeaderWindow)
        self.label.setObjectName("label")
        self.gridLayout.addWidget(self.label, 0, 0, 1, 1)
        self.fileNameField = QtWidgets.QLineEdit(FitsHeaderWindow)
        self.fileNameField.setReadOnly(True)
        self.fileNameField.setObjectName("fileNameField")
        self.gridLayout.addWidget(self.fileNameField, 0, 1, 1, 1)
        self.tabWidget = QtWidgets.QTabWidget(FitsHeaderWindow)
        self.tabWidget.setObjectName("tabWidget")
        self.gridLayout.addWidget(self.tabWidget, 1, 0, 1, 2)
        self.closeButton = QtWidgets.QPushButton(FitsHeaderWindow)
        self.closeButton.setMaximumSize(QtCore.QSize(100, 16777215))
        self.closeButton.setLayoutDirection(QtCore.Qt.LeftToRight)
        self.closeButton.setObjectName("closeButton")
        self.gridLayout.addWidget(self.closeButton, 2, 0, 1, 1)

        self.retranslateUi(FitsHeaderWindow)
        QtCore.QMetaObject.connectSlotsByName(FitsHeaderWindow)

    def retranslateUi(self, FitsHeaderWindow):
        _translate = QtCore.QCoreApplication.translate
        FitsHeaderWindow.setWindowTitle(_translate("FitsHeaderWindow", "FITS header info"))
        self.label.setText(_translate("FitsHeaderWindow", "FITS file name"))
        self.closeButton.setText(_translate("FitsHeaderWindow", "Close"))




import sys

import numpy as np
import pyqtgraph as pg
import pyqtgraph.functions as fn

from .fits import FITSUnit
from .functions import makeArrowPath


class From0ArrowItem(pg.ArrowItem):
    """
    Extends *pg.ArrowItem* so that the arrow's origin is not on the tip, but on
    the tail. This is necessary to make a good-looking compass.
    """
    def setStyle(self, **opts):
        self.opts.update(opts)

        opt = dict([(k, self.opts[k]) for k in ['headLen', 'tipAngle',
                                                'baseAngle', 'tailLen',
                                                'tailWidth']])
        self.path = makeArrowPath(opt['headLen'], opt['tipAngle'],
                                  opt['tailLen'], opt['tailWidth'])
        self.setPath(self.path)

        self.setPen(fn.mkPen(self.opts['pen']))
        self.setBrush(fn.mkBrush(self.opts['brush']))

        if self.opts['pxMode']:
            self.setFlags(self.flags() | self.ItemIgnoresTransformations)
        else:
            self.setFlags(self.flags() & ~self.ItemIgnoresTransformations)


class Compass:
    def __init__(self, plotItem):
        self.northArrow = From0ArrowItem(angle=90, tipAngle=30, baseAngle=10,
                                         headLen=20, tailLen=20, tailWidth=4,
                                         brush='r')
        self.northArrow.setPos(10, 10)
        self.northArrow.setZValue(1)
        self.eastArrow = From0ArrowItem(angle=0, tipAngle=30, baseAngle=10,
                                        headLen=20, tailLen=20, tailWidth=4,
                                        brush='b')
        self.eastArrow.setPos(10, 10)
        self.eastArrow.setZValue(1)
        self.hide()
        plotItem.addItem(self.northArrow)
        plotItem.addItem(self.eastArrow)

    def setFromFitsFile(self, fitsFile):
        self.northArrow.setStyle(angle=fitsFile.get_north_angle())
        self.northArrow.show()
        self.eastArrow.setStyle(angle=fitsFile.get_east_angle())
        self.eastArrow.show()

    def hide(self):
        self.northArrow.hide()
        self.eastArrow.hide()


class WCSAxes:
    def __init__(self):
        self.bottom = WCSAxis(orientation='bottom')
        self.top = WCSAxis(orientation='top')
        self.left = WCSAxis(orientation='left')
        self.right = WCSAxis(orientation='right')
        self.plotItem = pg.PlotItem(axisItems={
            'top': self.top,
            'bottom': self.bottom,
            'left': self.left,
            'right': self.right
        })
        # By default, top and right axes are not shown
        self.plotItem.showAxis('top', True)
        self.plotItem.showAxis('right', True)
        self.plotItem.showGrid(True, True)
        self.compass = Compass(self.plotItem)

    def setFitsFile(self, fitsFile):
        self.bottom.setFitsFile(fitsFile)
        self.top.setFitsFile(fitsFile)
        self.left.setFitsFile(fitsFile)
        self.right.setFitsFile(fitsFile)
        self.compass.setFromFitsFile(fitsFile)


class WCSAxis(pg.AxisItem):
    def __init__(self, orientation, fitsFile=None, **kwargs):
        super().__init__(orientation, **kwargs)
        self.fitsFile = fitsFile

    def setFitsFile(self, fitsFile):
        self.fitsFile = fitsFile

    def getAxisNumber(self):
        if self.orientation == 'bottom' or self.orientation == 'top':
            return 1
        else:
            return 2

    def toWcs(self, x: float) -> float:
        if self.orientation == 'bottom':
            return self.fitsFile.convert_to_wcs((x, 0))[0]
        elif self.orientation == 'top':
            N = self.fitsFile.pixel_count(self.getAxisNumber())
            return self.fitsFile.convert_to_wcs((x, N))[0]
        elif self.orientation == 'left':
            return self.fitsFile.convert_to_wcs((0, x))[1]
        elif self.orientation == 'right':
            N = self.fitsFile.pixel_count(self.getAxisNumber())
            return self.fitsFile.convert_to_wcs((N, x))[1]
        else:
            raise NotImplementedError('Unexpected orientation: '
                                      '%s' % self.orientation)

    def tickSpacing(self, minVal, maxVal, size):
        original = super().tickSpacing(minVal, maxVal, size)
        if self.fitsFile is None:
            return original
        ref = self.fitsFile.reference_pixel(self.getAxisNumber())
        return [(spacing, offset + ref) for spacing, offset in original]

    def tickStrings(self, values, scale, spacing):
        if self.fitsFile is None:
            return super().tickStrings(values, scale, spacing)
        ref = self.fitsFile.reference_pixel(self.getAxisNumber())
        unit = self.fitsFile.unit(self.getAxisNumber())
        ref_wcs = self.toWcs(ref)
        strings = []
        for v in values:
            coords = self.toWcs(v)
            if unit == FITSUnit.DEGREE:
                abs_coords = '%.4f°' % coords
                rel_arcsec = (coords - ref_wcs) * 3600
                if rel_arcsec == 0:
                    rel_arcsec = ''
                else:
                    rel_arcsec = "%.2e''" % rel_arcsec
            else:
                abs_coord = '%.4f %s' % (coords, unit)
                # Cannot convert to arc second, as unit is unknown!
                rel_arcsec = '%.2e %s' % ((coords - ref_wcs) * 3600, unit)
            # For top and right axis, only show the relative value in arcsec
            if self.orientation == 'top':
                string = '\n%s' % ('0' if rel_arcsec == '' else rel_arcsec)
            elif self.orientation == 'right':
                string = '0' if rel_arcsec == '' else rel_arcsec
            else:
                string = "%d\n%s\n%s" % (v - ref, abs_coords, rel_arcsec)
            if self.orientation == 'bottom':
                # Add space above to avoid overlapping of axis and labels
                string = '\n%s\n' % string
            strings.append(string)
        return strings



from pyqtgraph.widgets.PlotWidget import PlotWidget


class AstroWaveFormPlotWidget(PlotWidget):
    """
    Extends PlotWidget.
     - Add units and labels to plot
     - Flux unit is directly read from FITS file
    """
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.fitsFile = None

    def setFitsFile(self, fits):
        self.fitsFile = fits
        unit = self.fitsFile.wave_unit()
        if unit is not None:
            text = 'Wavelength (%s)' % unit
        else:
            text = 'Wavelength'
        self.setLabel('bottom', text=text)

    def updateAstroDisplay(self, show):
        if show and self.fitsFile is not None:
            unit = self.fitsFile.summed_flux_unit()
            if unit:
                text = 'Flux ({})'.format(unit)
            else:
                text = 'Flux'
            self.setLabel('left', text=text)
        else:
            self.showLabel('left', show=False)



from pyqtgraph.Qt import QtGui, QtWidgets

from .mainwindow_ui import Ui_MainWindow
from .AboutDialog import AboutDialog
from .AstroImageView import AstroImageView
from .FitsHeaderWindow import FitsHeaderWindow
from .MultiROI import ROIType

from .fits import FITS
from .settings import Settings


class MainWindow(QtGui.QMainWindow):
    def __init__(self):
        QtGui.QMainWindow.__init__(self)
        self.ui = Ui_MainWindow()
        self.ui.setupUi(self)
        self.setCentralWidget(self.ui.astroImageView)

        # QtDesigner does not support QActionGroup (radio button), so we handle
        # it outside of the generated UI file
        self.roiSelectGroup = QtWidgets.QActionGroup(self)
        self.roiSelectGroup.addAction(self.ui.actionDisable_ROI)
        self.roiSelectGroup.addAction(self.ui.actionRectangular_ROI)
        self.roiSelectGroup.addAction(self.ui.actionPolygonal_ROI)
        self.roiSelectGroup.addAction(self.ui.actionElliptic_ROI)
        self.roiSelectGroup.addAction(self.ui.actionSemi_automatic_ROI)

        # Populate "Recent Files" menu
        recentFiles = Settings.getRecentFiles()
        for i, path in enumerate(recentFiles):
            if i == 0:
                self.ui.action_NoRecentFile.setVisible(False)
            recentFileAction = QtWidgets.QAction(self)
            recentFileAction.setText("%i | %s" % (i, path))
            openFileFunction = self._createOpenFileFunction(path)
            recentFileAction.triggered.connect(openFileFunction)
            self.ui.menuRecent_Files.insertAction(self.ui.actionClear_Menu,
                                                  recentFileAction)
            if i == len(recentFiles) - 1:
                self.ui.menuRecent_Files.insertSeparator(
                    self.ui.actionClear_Menu)

        # Connect actions to event handlers
        self.ui.actionExit.triggered.connect(QtWidgets.qApp.quit)
        self.ui.actionOpen.triggered.connect(self._open_file)
        self.ui.actionClear_Menu.triggered.connect(self._clearRecentFiles)
        self.ui.actionSee_FITS_header.triggered.connect(
            self._openFitsHeaderDialog
        )
        self.roiSelectGroup.triggered.connect(self._selectROI)
        self.ui.actionAbout.triggered.connect(self._openAboutDialog)

        self.fitsFileStatusBarLabel = QtGui.QLabel()
        self.ui.statusbar.addPermanentWidget(self.fitsFileStatusBarLabel)
        self.dialog = None
        self.fitsFile = None

    def set_fits_file(self, file_path):
        Settings.addRecentFile(file_path)
        self.setWindowTitle('aflak - %s' % file_path)
        self.fitsFile = FITS(file_path)
        self.ui.astroImageView.set_fits_file(self.fitsFile)
        self.ui.menuFITS.setEnabled(True)
        self.fitsFileStatusBarLabel.setText(file_path)

    def _open_file(self):
        name, _file_type = QtGui.QFileDialog.getOpenFileName(self,
                                                             'Open File')
        # `name' is an empty string if the user pressed `Cancel' in the
        # open-file dialog
        if name != '':
            self.set_fits_file(name)

    def _createOpenFileFunction(self, path):
        def openFile():
            self.set_fits_file(path)
        return openFile

    def _clearRecentFiles(self):
        Settings.clearRecentFiles()
        self.ui.action_NoRecentFile.setVisible(True)
        for action in self.ui.menuRecent_Files.actions():
            if (action is not self.ui.action_NoRecentFile and
                    action is not self.ui.actionClear_Menu):
                self.ui.menuRecent_Files.removeAction(action)

    def _openFitsHeaderDialog(self):
        self.dialog = FitsHeaderWindow(self.fitsFile)
        self.dialog.show()

    def _selectROI(self, selectedAction):
        name = selectedAction.objectName()
        if name == 'actionDisable_ROI':
            self.ui.astroImageView.setROIType(None)
        elif name == 'actionElliptic_ROI':
            self.ui.astroImageView.setROIType(ROIType.ELLIPSE)
        elif name == 'actionPolygonal_ROI':
            self.ui.astroImageView.setROIType(ROIType.POLYGON)
        elif name == 'actionRectangular_ROI':
            self.ui.astroImageView.setROIType(ROIType.RECTANGLE)
        elif name == 'actionSemi_automatic_ROI':
            self.ui.astroImageView.setROIType(ROIType.SEMIAUTOMATIC)
        else:
            raise NotImplementedError('Unknown action name: %s' % name)

    def _openAboutDialog(self):
        AboutDialog().exec_()



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'aflak/mainwindow.ui'
#
# Created by: PyQt5 UI code generator 5.9.2
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_MainWindow(object):
    def setupUi(self, MainWindow):
        MainWindow.setObjectName("MainWindow")
        MainWindow.resize(800, 600)
        self.astroImageView = AstroImageView(MainWindow)
        self.astroImageView.setObjectName("astroImageView")
        self.menubar = QtWidgets.QMenuBar(MainWindow)
        self.menubar.setGeometry(QtCore.QRect(0, 0, 800, 34))
        self.menubar.setObjectName("menubar")
        self.menuFile = QtWidgets.QMenu(self.menubar)
        self.menuFile.setObjectName("menuFile")
        self.menuRecent_Files = QtWidgets.QMenu(self.menuFile)
        self.menuRecent_Files.setObjectName("menuRecent_Files")
        self.menuFITS = QtWidgets.QMenu(self.menubar)
        self.menuFITS.setEnabled(False)
        self.menuFITS.setObjectName("menuFITS")
        self.menuHelp = QtWidgets.QMenu(self.menubar)
        self.menuHelp.setObjectName("menuHelp")
        self.menuAnalyze = QtWidgets.QMenu(self.menubar)
        self.menuAnalyze.setObjectName("menuAnalyze")
        MainWindow.setMenuBar(self.menubar)
        self.statusbar = QtWidgets.QStatusBar(MainWindow)
        self.statusbar.setObjectName("statusbar")
        MainWindow.setStatusBar(self.statusbar)
        self.actionOpen = QtWidgets.QAction(MainWindow)
        self.actionOpen.setObjectName("actionOpen")
        self.actionExit = QtWidgets.QAction(MainWindow)
        self.actionExit.setObjectName("actionExit")
        self.actionSee_FITS_header = QtWidgets.QAction(MainWindow)
        self.actionSee_FITS_header.setObjectName("actionSee_FITS_header")
        self.actionAbout = QtWidgets.QAction(MainWindow)
        self.actionAbout.setObjectName("actionAbout")
        self.actionRectangular_ROI = QtWidgets.QAction(MainWindow)
        self.actionRectangular_ROI.setCheckable(True)
        self.actionRectangular_ROI.setObjectName("actionRectangular_ROI")
        self.actionElliptic_ROI = QtWidgets.QAction(MainWindow)
        self.actionElliptic_ROI.setCheckable(True)
        self.actionElliptic_ROI.setObjectName("actionElliptic_ROI")
        self.actionPolygonal_ROI = QtWidgets.QAction(MainWindow)
        self.actionPolygonal_ROI.setCheckable(True)
        self.actionPolygonal_ROI.setObjectName("actionPolygonal_ROI")
        self.actionSemi_automatic_ROI = QtWidgets.QAction(MainWindow)
        self.actionSemi_automatic_ROI.setCheckable(True)
        self.actionSemi_automatic_ROI.setObjectName("actionSemi_automatic_ROI")
        self.actionDisable_ROI = QtWidgets.QAction(MainWindow)
        self.actionDisable_ROI.setCheckable(True)
        self.actionDisable_ROI.setChecked(True)
        self.actionDisable_ROI.setObjectName("actionDisable_ROI")
        self.action_NoRecentFile = QtWidgets.QAction(MainWindow)
        self.action_NoRecentFile.setEnabled(False)
        self.action_NoRecentFile.setObjectName("action_NoRecentFile")
        self.actionClear_Menu = QtWidgets.QAction(MainWindow)
        self.actionClear_Menu.setObjectName("actionClear_Menu")
        self.menuRecent_Files.addAction(self.action_NoRecentFile)
        self.menuRecent_Files.addSeparator()
        self.menuRecent_Files.addAction(self.actionClear_Menu)
        self.menuFile.addAction(self.actionOpen)
        self.menuFile.addAction(self.menuRecent_Files.menuAction())
        self.menuFile.addAction(self.actionExit)
        self.menuFITS.addAction(self.actionSee_FITS_header)
        self.menuHelp.addAction(self.actionAbout)
        self.menuAnalyze.addSeparator().setText("ROI")
        self.menuAnalyze.addAction(self.actionDisable_ROI)
        self.menuAnalyze.addAction(self.actionElliptic_ROI)
        self.menuAnalyze.addAction(self.actionPolygonal_ROI)
        self.menuAnalyze.addAction(self.actionRectangular_ROI)
        self.menuAnalyze.addAction(self.actionSemi_automatic_ROI)
        self.menubar.addAction(self.menuFile.menuAction())
        self.menubar.addAction(self.menuFITS.menuAction())
        self.menubar.addAction(self.menuAnalyze.menuAction())
        self.menubar.addAction(self.menuHelp.menuAction())

        self.retranslateUi(MainWindow)
        QtCore.QMetaObject.connectSlotsByName(MainWindow)

    def retranslateUi(self, MainWindow):
        _translate = QtCore.QCoreApplication.translate
        MainWindow.setWindowTitle(_translate("MainWindow", "aflak"))
        self.menuFile.setTitle(_translate("MainWindow", "File"))
        self.menuRecent_Files.setTitle(_translate("MainWindow", "Recent Files"))
        self.menuFITS.setTitle(_translate("MainWindow", "FITS"))
        self.menuHelp.setTitle(_translate("MainWindow", "Help"))
        self.menuAnalyze.setTitle(_translate("MainWindow", "Analyze"))
        self.actionOpen.setText(_translate("MainWindow", "Open FITS"))
        self.actionOpen.setShortcut(_translate("MainWindow", "Ctrl+O"))
        self.actionExit.setText(_translate("MainWindow", "Exit"))
        self.actionExit.setShortcut(_translate("MainWindow", "Ctrl+Q"))
        self.actionSee_FITS_header.setText(_translate("MainWindow", "See FITS header"))
        self.actionSee_FITS_header.setShortcut(_translate("MainWindow", "Ctrl+H"))
        self.actionAbout.setText(_translate("MainWindow", "About"))
        self.actionRectangular_ROI.setText(_translate("MainWindow", "Rectangular ROI"))
        self.actionElliptic_ROI.setText(_translate("MainWindow", "Elliptic ROI"))
        self.actionPolygonal_ROI.setText(_translate("MainWindow", "Polygonal ROI"))
        self.actionSemi_automatic_ROI.setText(_translate("MainWindow", "Semi-automatic ROI"))
        self.actionDisable_ROI.setText(_translate("MainWindow", "Disable ROI"))
        self.action_NoRecentFile.setText(_translate("MainWindow", "(None)"))
        self.actionClear_Menu.setText(_translate("MainWindow", "Clear Menu"))

from .AstroImageView import AstroImageView



from pyqtgraph.Qt import QtWidgets

from .FitsHeaderForm import FitsHeaderForm
from .FitsHeaderWindow_ui import Ui_FitsHeaderWindow


class FitsHeaderWindow(QtWidgets.QDialog):
    def __init__(self, fitsFile):
        super().__init__()
        self.ui = Ui_FitsHeaderWindow()
        self.ui.setupUi(self)
        self.ui.closeButton.clicked.connect(self.close)
        self.ui.fileNameField.setText(fitsFile.name)
        for i, hdu in enumerate(fitsFile.hdulist):
            tab = QtWidgets.QWidget()
            gridLayout = QtWidgets.QGridLayout(tab)
            tabContent = FitsHeaderForm(tab)
            tabContent.setHdu(hdu)
            gridLayout.addWidget(tabContent)
            self.ui.tabWidget.addTab(tab, "")
            self.ui.tabWidget.setTabText(
                self.ui.tabWidget.indexOf(tab), "[%d] %s" % (i, hdu.name)
            )



import enum

from astropy import wcs
from astropy.io import fits
import numpy as np


class FITSUnit(enum.Enum):
    DEGREE = 1


class FITS:
    def __init__(self, name, **kwargs):
        self.hdulist = fits.open(name, **kwargs)
        self.name = name
        self.wcs = wcs.WCS(self.hdulist['FLUX'].header)
        print('Opened file: {}'.format(name))
        print(self.hdulist.info())

    def wave(self):
        return self.hdulist['WAVE'].data

    def flux(self):
        return self.hdulist['FLUX'].data

    def wave_unit(self):
        """
        Return the unit of the waveform in a suitable format to be displayed on
        a plot, if possible.
        """
        if 'CUNIT3' in self.hdulist['FLUX'].header:
            raw_unit = self.hdulist['FLUX'].header['CUNIT3']
            if raw_unit == 'Angstrom':
                return 'Å'
            else:
                return raw_unit

    def flux_unit(self):
        if 'BUNIT' in self.hdulist['FLUX'].header:
            return self.hdulist['FLUX'].header['BUNIT']

    def summed_flux_unit(self):
        flux_unit = self.flux_unit()
        if flux_unit is not None:
            return flux_unit.replace("/Ang", "")

    def reference_pixel(self, axis: int) -> float:
        key = 'CRPIX{}'.format(axis)
        if key in self.hdulist['FLUX'].header:
            # In FITS files, indexing start from 1 (like FORTRAN)
            return float(self.hdulist['FLUX'].header[key]) - 1
        else:
            raise KeyError('Unknown axis "{}". '
                           'Key not found: "{}"'.format(axis, key))

    def pixel_count(self, axis: int) -> float:
        key = 'NAXIS{}'.format(axis)
        if key in self.hdulist['FLUX'].header:
            return float(self.hdulist['FLUX'].header[key])
        else:
            raise KeyError('Unknown axis "{}". '
                           'Key not found: "{}"'.format(axis, key))

    def convert_to_wcs(self, pixel: (float, float)) -> [float, float]:
        return self.wcs.all_pix2world([[pixel[0], pixel[1], 1]], 1)[0, 0:2]

    def unit(self, axis: int) -> FITSUnit or str:
        key = 'CUNIT{}'.format(axis)
        if key in self.hdulist['FLUX'].header:
            unit = self.hdulist['FLUX'].header[key]
            if unit == 'deg':
                return FITSUnit.DEGREE
            else:
                return unit
        else:
            raise KeyError('Unknown axis "{}". '
                           'Key not found: "{}"'.format(axis, key))

    def get_north_angle(self) -> float:
        ref1 = self.reference_pixel(1)
        ref2 = self.reference_pixel(2)
        ref_wcoords = self.wcs.all_pix2world([[ref1, ref2, 1]], 1)[0]
        northern_point = self.wcs.all_world2pix([[ref_wcoords[0],
                                                  ref_wcoords[1] + 1,
                                                  ref_wcoords[2]]], 1)[0, 0:2]
        northern_vector = northern_point - np.array([ref1, ref2])
        return np.arctan2(northern_point[1], northern_point[0])

    def get_east_angle(self) -> float:
        ref1 = self.reference_pixel(1)
        ref2 = self.reference_pixel(2)
        ref_wcoords = self.wcs.all_pix2world([[ref1, ref2, 1]], 1)[0]
        eastern_point = self.wcs.all_world2pix([[ref_wcoords[0] - 1,
                                                 ref_wcoords[1],
                                                 ref_wcoords[2]]], 1)[0, 0:2]
        eastern_vector = eastern_point - np.array([ref1, ref2])
        return np.arctan2(eastern_vector[1], eastern_vector[0])



import numpy as np

from pyqtgraph.Qt import QtGui


def north(node):
    return (node[0], node[1] - 1)


def south(node):
    return (node[0], node[1] + 1)


def west(node):
    return (node[0] - 1, node[1])


def east(node):
    return (node[0] + 1, node[1])


def toIntTuple(arraylike):
    return int(arraylike[0]), int(arraylike[1])


def floodfill(img, node, predicate):
    firstNode = toIntTuple(node)
    mask = np.zeros(img.shape, dtype=bool)
    mask[firstNode] = True
    queue = [firstNode]
    while len(queue) > 0:
        node = queue.pop()
        N = north(node)
        S = south(node)
        W = west(node)
        E = east(node)
        for nextNode in [N, S, W, E]:
            if (nextNode[0] < 0 or nextNode[0] >= img.shape[0] or
                    nextNode[1] < 0 or nextNode[1] >= img.shape[1]):
                continue
            if (not mask[nextNode]) and predicate(img[nextNode]):
                mask[nextNode] = True
                queue.append(nextNode)
    return mask


def getPath(mask):
    """
    Return the path of the countour of a boolean numpy 2d-array.
    """
    path = QtGui.QPainterPath()
    for i in range(mask.shape[0]):
        for j in range(mask.shape[1]):
            # Draw vertical lines
            if i == 0:  # Border first
                if mask[i, j]:
                    path.moveTo(i, j)
                    path.lineTo(i, j + 1)
            elif i == mask.shape[0] - 1:
                if mask[i, j]:
                    path.moveTo(i + 1, j)
                    path.lineTo(i + 1, j + 1)
            else:
                if mask[i, j] != mask[i + 1, j]:
                    path.moveTo(i + 1, j)
                    path.lineTo(i + 1, j + 1)
            # Draw horizontal lines
            if j == 0:  # Border first
                if mask[i, j]:
                    path.moveTo(i, j)
                    path.lineTo(i + 1, j)
            elif j == mask.shape[1] - 1:
                if mask[i, j]:
                    path.moveTo(i, j + 1)
                    path.lineTo(i + 1, j + 1)
            else:
                if mask[i, j] != mask[i, j + 1]:
                    path.moveTo(i, j + 1)
                    path.lineTo(i + 1, j + 1)
    return path


def makeArrowPath(headLen, tipAngle, tailLen, tailWidth):
    """
    Make arrow path for which origin is the extremity of the arrow's tail.
    """
    headWidth = headLen * np.tan(tipAngle * 0.5 * np.pi/180.)
    path = QtGui.QPainterPath()
    path.moveTo(0, 0)
    path.lineTo(0, -tailWidth/2)
    path.lineTo(-tailLen, -tailWidth/2)
    path.lineTo(-tailLen, -tailWidth/2 - headWidth/2)
    path.lineTo(-tailLen - headLen, 0)
    path.lineTo(-tailLen, tailWidth/2 + headWidth/2)
    path.lineTo(-tailLen, tailWidth/2)
    path.lineTo(0, tailWidth/2)
    path.lineTo(0, 0)
    return path



from pyqtgraph.Qt import QtWidgets

from .FitsHeaderForm_ui import Ui_Form


class FitsHeaderForm(QtWidgets.QWidget):
    def __init__(self, *args, **kwargs):
        QtWidgets.QWidget.__init__(self, *args, **kwargs)
        self.ui = Ui_Form()
        self.ui.setupUi(self)

    def setHdu(self, hdu):
        self.ui.plainTextEdit.setPlainText(repr(hdu.header))



import threading

import numpy as np
import pyqtgraph as pg
from pyqtgraph.Qt import QtCore, QtGui
from pyqtgraph.graphicsItems.ImageItem import ImageItem
from pyqtgraph.graphicsItems.InfiniteLine import InfiniteLine
from pyqtgraph.graphicsItems.LinearRegionItem import LinearRegionItem
from pyqtgraph.graphicsItems.ViewBox import ViewBox

from .AstroImageView_ui import Ui_Form
from .MultiROI import MultiROI, ROIType
from .WCSAxis import WCSAxes


class AstroImageView(pg.ImageView):
    """
    Extends ImageView class.
     - Replace default HistogramLUTWidget with our AstroHistogramLUTWidget.
     - Replace PlotWidget for roiPlot (defined inside UI class) by
       AstroWaveFormPlotWidget
     - Delete roiBtn
    """
    def __init__(self, parent=None, name="ImageView", view=None,
                 imageItem=None, *args):
        QtGui.QWidget.__init__(self, parent, *args)
        self.levelMax = 4096
        self.levelMin = 0
        self.name = name
        self.image = None
        self.axes = {}
        self.imageDisp = None
        self.ui = Ui_Form()
        self.ui.setupUi(self)
        self.scene = self.ui.graphicsView.scene()

        self.ignoreTimeLine = False

        self.wcsAxes = WCSAxes()
        if view is None:
            self.view = self.wcsAxes.plotItem
        else:
            self.view = view
        self.ui.graphicsView.setCentralItem(self.view)
        self.view.setAspectLocked(True)
        self.view.invertY()

        if imageItem is None:
            self.imageItem = ImageItem()
        else:
            self.imageItem = imageItem
        self.view.addItem(self.imageItem)
        self.currentIndex = 0

        self.ui.histogram.setImageItem(self.imageItem)

        self.menu = None

        self._hideSlider()

        self.ui.normGroup.hide()

        self.roiEnabled = False
        self.roi = MultiROI()
        self.roi.setZValue(20)
        self.view.addItem(self.roi)
        self.roi.hide()
        self.normRoi = MultiROI()
        self.normRoi.setPen('y')
        self.normRoi.setZValue(20)
        self.view.addItem(self.normRoi)
        self.normRoi.hide()
        self.roiCurve = self.ui.roiPlot.plot()
        self.timeLine = InfiniteLine(0, movable=True)
        self.timeLine.setPen((255, 255, 0, 200))
        self.timeLine.setZValue(1)
        self.ui.roiPlot.addItem(self.timeLine)
        self.ui.splitter.setSizes([self.height()-35, 35])
        self.ui.roiPlot.hideAxis('left')

        self.keysPressed = {}
        self.playTimer = QtCore.QTimer()
        self.playRate = 0
        self.lastPlayTime = 0

        self.normRgn = LinearRegionItem()
        self.normRgn.setZValue(0)
        self.ui.roiPlot.addItem(self.normRgn)
        self.normRgn.hide()

        # wrap functions from view box
        for fn in ['addItem', 'removeItem']:
            setattr(self, fn, getattr(self.view, fn))

        # wrap functions from histogram
        for fn in ['setHistogramRange', 'autoHistogramRange', 'getLookupTable',
                   'getLevels']:
            setattr(self, fn, getattr(self.ui.histogram, fn))

        self.ui.horizontalSlider.valueChanged.connect(self._sliderValueChanged)
        self.ui.sliderValueSpinBox.valueChanged.connect(self._spinBoxChanged)
        self.timeLine.sigPositionChanged.connect(self.timeLineChanged)
        self.roi.sigRegionChanged.connect(self.roiChanged)

        self.ui.menuBtn.clicked.connect(self.menuClicked)
        self.ui.normDivideRadio.clicked.connect(self.normRadioChanged)
        self.ui.normSubtractRadio.clicked.connect(self.normRadioChanged)
        self.ui.normOffRadio.clicked.connect(self.normRadioChanged)
        self.ui.normROICheck.clicked.connect(self.updateNorm)
        self.ui.normFrameCheck.clicked.connect(self.updateNorm)
        self.ui.normTimeRangeCheck.clicked.connect(self.updateNorm)
        self.playTimer.timeout.connect(self.timeout)

        self.normProxy = pg.SignalProxy(self.normRgn.sigRegionChanged,
                                        slot=self.updateNorm)
        self.normRoi.sigRegionChangeFinished.connect(self.updateNorm)

        self.ui.roiPlot.registerPlot(self.name + '_ROI')
        self.view.register(self.name)

        self.noRepeatKeys = [QtCore.Qt.Key_Right, QtCore.Qt.Key_Left,
                             QtCore.Qt.Key_Up, QtCore.Qt.Key_Down,
                             QtCore.Qt.Key_PageUp, QtCore.Qt.Key_PageDown]

        # initialize roi plot to correct shape / visibility
        self.setROIType(None)

    def set_fits_file(self, fitsFile):
        self.wcsAxes.setFitsFile(fitsFile)
        flux = fitsFile.flux()
        wave = fitsFile.wave()
        self.setImage(flux, xvals=wave)
        self.ui.roiPlot.setFitsFile(fitsFile)
        # TODO: Autorange the image after it is drawn.
        # This is a work around for what seems to be a bug in the
        # *pg.ArrowItem*: When arrows are shown in the canvas, the default
        # range of the axis becomes larger. Arrows are included from *WCSAxis*.
        timer = threading.Timer(0.1, lambda: self.view.vb.menu.autoRange())
        timer.start()

    def setImage(self, img, autoRange=True, autoLevels=True, levels=None,
                 axes=None, xvals=None, pos=None, scale=None, transform=None,
                 autoHistogramRange=True):
        """
        Reimplement setImage without using roiBtn
        """
        if hasattr(img, 'implements') and img.implements('MetaArray'):
            img = img.asarray()

        if not isinstance(img, np.ndarray):
            required = ['dtype', 'max', 'min', 'ndim', 'shape', 'size']
            if not all([hasattr(img, attr) for attr in required]):
                raise TypeError("Image must be NumPy array or any object "
                                "that provides compatible attributes/methods:"
                                "\n  %s" % str(required))

        self.image = img
        self.imageDisp = None

        if axes is None:
            if self.imageItem.axisOrder == 'col-major':
                x, y = (0, 1)
            else:
                x, y = (1, 0)

            if img.ndim == 2:
                self.axes = {'t': None, 'x': x, 'y': y, 'c': None}
            elif img.ndim == 3:
                # Ambiguous case; make a guess
                if img.shape[2] <= 4:
                    self.axes = {'t': None, 'x': x, 'y': y, 'c': 2}
                else:
                    self.axes = {'t': 0, 'x': x+1, 'y': y+1, 'c': None}
            elif img.ndim == 4:
                # Even more ambiguous; just assume the default
                self.axes = {'t': 0, 'x': x+1, 'y': y+1, 'c': 3}
            else:
                raise Exception("Can not interpret image with dimensions %s" %
                                str(img.shape))
        elif isinstance(axes, dict):
            self.axes = axes.copy()
        elif isinstance(axes, list) or isinstance(axes, tuple):
            self.axes = {}
            for i in range(len(axes)):
                self.axes[axes[i]] = i
        else:
            raise Exception("Can not interpret axis specification %s. "
                            "Must be like {'t': 2, 'x': 0, 'y': 1} or "
                            "('t', 'x', 'y', 'c')" % str(axes))

        for x in ['t', 'x', 'y', 'c']:
            self.axes[x] = self.axes.get(x, None)
        axes = self.axes

        if xvals is not None:
            self.tVals = xvals
        elif axes['t'] is not None:
            if hasattr(img, 'xvals'):
                try:
                    self.tVals = img.xvals(axes['t'])
                except Exception:
                    self.tVals = np.arange(img.shape[axes['t']])
            else:
                self.tVals = np.arange(img.shape[axes['t']])

        self.currentIndex = 0
        self.updateImage(autoHistogramRange=autoHistogramRange)
        if levels is None and autoLevels:
            self.autoLevels()
        # this does nothing since getProcessedImage sets these values again.
        if levels is not None:
            self.setLevels(*levels)

        if self.roiEnabled:
            self.roiChanged()

        if self.axes['t'] is not None:
            self.ui.roiPlot.setXRange(self.tVals.min(), self.tVals.max())
            self.timeLine.setValue(0)
            if len(self.tVals) > 1:
                start = self.tVals.min()
                stop = (self.tVals.max() +
                        abs(self.tVals[-1] - self.tVals[0]) * 0.02)
            elif len(self.tVals) == 1:
                start = self.tVals[0] - 0.5
                stop = self.tVals[0] + 0.5
            else:
                start = 0
                stop = 1
            for s in [self.timeLine, self.normRgn]:
                s.setBounds([start, stop])

        self.imageItem.resetTransform()
        if scale is not None:
            self.imageItem.scale(*scale)
        if pos is not None:
            self.imageItem.setPos(*pos)
        if transform is not None:
            self.imageItem.setTransform(transform)

        if autoRange:
            self.autoRange()
        self._updateRoiPlot()

    def setROIType(self, roiType):
        if roiType is None:
            self.roiEnabled = False
        else:
            self.roiEnabled = True
            if roiType == ROIType.SEMIAUTOMATIC:
                self._showSlider()
            else:
                self._hideSlider()
            self.roi.setROIType(roiType)
        self._updateRoiPlot()

    def _updateRoiPlot(self):
        showRoiPlot = False
        if self.roiEnabled:
            showRoiPlot = True
            self.roi.show()
            self.ui.roiPlot.setMouseEnabled(True, True)
            self.ui.splitter.setSizes([self.height()*0.6, self.height()*0.4])
            self.roiCurve.show()
            self.roiChanged()
            self.ui.roiPlot.showAxis('left')
        else:
            self.roi.hide()
            self.ui.roiPlot.setMouseEnabled(False, False)
            self.roiCurve.hide()
            self.ui.roiPlot.hideAxis('left')

        if self.hasTimeAxis():
            showRoiPlot = True
            mn = self.tVals.min()
            mx = self.tVals.max()
            self.ui.roiPlot.setXRange(mn, mx, padding=0.01)
            self.timeLine.show()
            self.timeLine.setBounds([mn, mx])
            self.ui.roiPlot.show()
            if not self.roiEnabled:
                self.ui.splitter.setSizes([self.height()-35, 35])
        else:
            self.timeLine.hide()

        self.ui.roiPlot.setVisible(showRoiPlot)

    def _hideSlider(self):
        self.ui.horizontalSliderLabel.hide()
        self.ui.horizontalSlider.hide()
        self.ui.sliderValueSpinBox.hide()

    def _showSlider(self):
        self.ui.horizontalSliderLabel.show()
        self.ui.horizontalSlider.show()
        self.ui.sliderValueSpinBox.show()

    def _sliderValueChanged(self, value):
        spinBoxVal = value / 100.0
        self.ui.sliderValueSpinBox.setValue(spinBoxVal)
        self.roi.setSliderValue(spinBoxVal)

    def _spinBoxChanged(self, value):
        self.ui.horizontalSlider.setValue(int(value * 100))
        self.roi.setSliderValue(value)



import io
import re
from os.path import dirname
from os.path import join

from setuptools import setup


def read(*names, **kwargs):
    return io.open(
        join(dirname(__file__), *names),
        encoding=kwargs.get('encoding', 'utf8')
    ).read()


def get_version():
    f = open('aflak/__init__.py')
    version_re = re.compile("^__version__ = '(\d.\d.\d)'$")
    for line in f:
        match = version_re.match(line)
        if match:
            return match[1]
    raise "Version not found"


setup(name='aflak',
      version=get_version(),
      python_requires='>=3',
      description='Advanced Framework for Learning Astrophysical Knowledge',
      long_description='%s\n%s' % (
        re.compile('^.. start-badges.*^.. end-badges',
                   re.M | re.S).sub('',  read('README.rst')),
        re.sub(':[a-z]+:`~?(.*?)`', r'``\1``', read('CHANGELOG.rst'))
      ),
      author='Malik Olivier Boussejra',
      author_email='malik@boussejra.com',
      url='https://github.com/malikolivier/aflak',
      license='GPLv3',
      packages=['aflak'],
      entry_points={
          'console_scripts': [
              'aflak = aflak.aflak:main',
          ]
      },
      install_requires=[
          'astropy',
          'pyqtgraph',
          'PyQt5',
      ],
      classifiers=[
          'Development Status :: 1 - Planning',
          'Environment :: Console',
          'Environment :: X11 Applications :: Qt',
          'Intended Audience :: End Users/Desktop',
          'Intended Audience :: Education',
          'Intended Audience :: Science/Research',
          'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',
          'Operating System :: POSIX :: Linux',
          'Programming Language :: Python :: 3.6',
          'Topic :: Multimedia :: Graphics',
          'Topic :: Scientific/Engineering :: Astronomy',
          'Topic :: Scientific/Engineering :: Physics',
          'Topic :: Scientific/Engineering :: Visualization'
      ])



import datetime
import os
import sys

from pyqtgraph.Qt import QtCore


QtCore.QCoreApplication.setOrganizationName('aflak')
QtCore.QCoreApplication.setOrganizationDomain('aflak.jp')
QtCore.QCoreApplication.setApplicationName('aflak')


class Settings:
    settings = QtCore.QSettings()

    @classmethod
    def addRecentFile(cls, filePath):
        files = cls.settings.value('RecentFiles')
        if type(files) is not list:
            files = []
        absolutePath = os.path.join(os.getcwd(), filePath)
        files = [f for f in files if f['path'] != absolutePath]
        files.append({
            'time': datetime.datetime.now(),
            'path': absolutePath
        })
        cls.settings.setValue('RecentFiles', files)

    @classmethod
    def getRecentFiles(cls):
        files = cls.settings.value('RecentFiles')
        if type(files) is not list:
            return []
        else:
            try:
                files = sorted(files, key=lambda f: f['time'], reverse=True)
                return [f['path'] for f in files]
            except TypeError as e:
                # In case memory on disk got corrupted, reset memory
                print('Unexpected error on parsing RecentFiles',
                      file=sys.stderr)
                print(e, file=sys.stderr)
                cls.clearRecentFiles()
                return []

    @classmethod
    def clearRecentFiles(cls):
        cls.settings.setValue('RecentFiles', [])



import argparse

from pyqtgraph.Qt import QtGui

from aflak.__init__ import __version__
import aflak.mainwindow


def main():
    parser = argparse.ArgumentParser(description='Provide FITS file as input')
    parser.add_argument('fits', metavar='fits-file', nargs='?',
                        help='FITS file to open')
    parser.add_argument('--version', action='version',
                        version='aflak {}'.format(__version__))

    args = parser.parse_args()

    app = QtGui.QApplication([])
    main = aflak.mainwindow.MainWindow()
    if args.fits is not None:
        main.set_fits_file(args.fits)

    main.show()
    return QtGui.QApplication.instance().exec_()



from pyqtgraph.Qt import QtWidgets

from . import __version__
from .AboutDialog_ui import Ui_AboutDialog


class AboutDialog(QtWidgets.QDialog):
        def __init__(self):
            super().__init__()
            self.ui = Ui_AboutDialog()
            self.ui.setupUi(self)
            self.ui.closeButton.clicked.connect(self.close)
            self.ui.versionLabel.setText(__version__)
